import React, { useState, useEffect, useCallback } from 'react';
import {
  message,
  notification,
  Form,
  Spin,
  Tree,
  Tooltip,
  Modal,
  Input,
  Button,
} from 'antd';
import {
  FileAddTwoTone,
  FolderAddTwoTone,
  EditTwoTone,
  DeleteTwoTone,
} from '@ant-design/icons';
import { getTree, addMenuApi, getRoleList } from '../../services/appConfig/api';
import ListCardItem from '../orgnazation/listCardItem';
import styles from './AppMenu.less';

const SevenParams = () => {
  const [treeLoading, setTreeLoading] = useState(true);

  const [treeData, setTreeData] = useState([]); // 菜单树
  const [rolelist, setRolelist] = useState([]);

  const [menuID, setMenuID] = useState('');
  const [menuTitle, setMenuTitle] = useState('菜单管理');

  const [addMenuVisible, setAddMenuVisible] = useState(false);
  const [addMenuGroupVisible, setAddMenuGroupVisible] = useState(false);
  const [tipVisible, setTipVisible] = useState(false);
  // const [editMenuVisible, setEditMenuVisible] = useState(false);
  const [deleteMenuVisible, setDeleteMenuVisible] = useState(false);
  const [addMenuForm] = Form.useForm();
  const [addMenuGroupForm] = Form.useForm();

  const getRoleValueCallback = useCallback((value, index) => {
    // roleValueList[index] = value;
    // setRoleValueList({ ...roleValueList });
  }, []);

  // 获取菜单树
  useEffect(() => {
    updateTrees();
  }, []);

  const updateTrees = () => {
    setTreeLoading(true);
    getTree()
      .then(res => {
        if (res.length > 0) {
          setTreeLoading(false);
          setTreeData(res[0].children[0].children[2].children);
          console.log(res[0].children[0].children[2].children);
        }
      })
      .catch(err => {
        setTreeLoading(false);
        message.error(err);
      });
  };

  // 获取角色列表
  useEffect(() => {
    getRoleList()
      .then(res => {
        if (res && res.list) {
          setRolelist(res.list);
        } else {
          notification.error({
            message: '获取失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        message.error(err);
      });
  }, []);

  // 渲染菜单树
  const mapTree = menu => {
    const haveChildren =
      Array.isArray(menu.children) && menu.children.length > 0;
    return {
      title: (
        <>
          <span>{menu.text}</span>
          <div className={styles.iconWraper1}>
            {menu.menuType === 'MiniAppMenuGroup' && (
              <>
                <Tooltip title="添加菜单组">
                  <FolderAddTwoTone
                    onClick={e => addMenuGroup(e, menu.text, menu.menuID)}
                  />
                </Tooltip>
              </>
            )}
            {menu.menuType === 'MiniAppMenuGroupTwo' && (
              <>
                <Tooltip title="添加菜单">
                  <FileAddTwoTone
                    onClick={e => addMenu(e, menu.text, menu.menuID)}
                  />
                </Tooltip>
              </>
            )}
            <Tooltip title="编辑当前菜单">
              <EditTwoTone onClick={e => editMenu(e, menu.text, menu.menuID)} />
            </Tooltip>
            <Tooltip title="删除当前菜单">
              <DeleteTwoTone
                onClick={e => deleteMenu(e, menu.text, menu.menuID)}
              />
            </Tooltip>
          </div>
        </>
      ),
      key: menu.menuID,
      // icon: <SolutionOutlined />,
      // 判断它是否存在子集，若果存在就进行再次进行遍历操作，知道不存在子集便对其他的元素进行操作
      children: haveChildren ? menu.children.map(i => mapTree(i)) : [],
    };
  };

  // 左侧目录树相关操作
  const addMenu = (e, title, id) => {
    setAddMenuVisible(true);
    setMenuTitle(`在${title}下添加菜单`);
    setMenuID(id);
    addMenuForm.setFieldsValue({
      menuName: '',
      shortName: '',
      imageUrl: '',
      pageUrl: '',
      funParam: '',
    });
  };
  const addMenuGroup = (e, title, id) => {
    setAddMenuGroupVisible(true);
    setMenuTitle(`在${title}下添加菜单组`);
    setMenuID(id);
  };
  const editMenu = (e, title, id) => {
    setAddMenuVisible(true);
    setMenuTitle(`编辑${title}`);
    setMenuID(id);
    addMenuApi(`/Cityinterface/rest/services/OMS.svc/M_GetMenu`, {
      _version: 9999,
      _dc: new Date().getTime(),
      menuID: id,
    })
      .then(res => {
        if (res.success) {
          addMenuForm.setFieldsValue({
            menuName: res.menuName,
            menuAlias: res.menuAlias,
            menuIcon: res.menuIcon,
            relatedRoleList: res.relatedRoleList || [],
          });
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        message.error(err);
      });
  };
  const deleteMenu = (e, title, id) => {
    setDeleteMenuVisible(true);
    setMenuTitle(`删除菜单${title}`);
    setMenuID(id);
  };
  // 提交添加菜单
  const submitAddMenu = () => {
    console.log(menuID);
    let menuName = addMenuForm.getFieldValue('menuName');
    let shortName = addMenuForm.getFieldValue('shortName');
    let imageUrl = addMenuForm.getFieldValue('imageUrl');
    let pageUrl = addMenuForm.getFieldValue('pageUrl');
    let funParam = addMenuForm.getFieldValue('funParam');
    if (menuName && shortName && imageUrl && pageUrl) {
      addMenuApi(menuID, menuName, shortName, imageUrl, pageUrl, funParam)
        .then(res => {
          if (res.success) {
            setAddMenuVisible(false);
            notification.success({
              message: '提交成功',
            });
            updateTrees();
          } else {
            notification.error({
              message: '提交失败',
              description: res.message,
            });
          }
        })
        .catch(err => {
          message.error(err);
        });
    } else {
      setTipVisible(true);
    }
  };
  // 提交添加菜单组
  const submitAddMenuGroup = () => {
    let menuName = addMenuGroupForm.getFieldValue('menuName');
    let shortName = addMenuGroupForm.getFieldValue('shortName');
    let imageUrl = addMenuGroupForm.getFieldValue('imageUrl');
    let pageUrl = addMenuGroupForm.getFieldValue('pageUrl');
    let funParam = addMenuGroupForm.getFieldValue('funParam');
    if (menuName && shortName && imageUrl && pageUrl) {
      addMenuApi(menuID, menuName, shortName, imageUrl, pageUrl, funParam)
        .then(res => {
          if (res.success) {
            setAddMenuGroupVisible(false);
            notification.success({
              message: '提交成功',
            });
            updateTrees();
          } else {
            notification.error({
              message: '提交失败',
              description: res.message,
            });
          }
        })
        .catch(err => {
          message.error(err);
        });
    } else {
      setTipVisible(true);
    }
  };
  // 提交删除菜单
  const submitDeleteMenu = () => {
    addMenuApi(`/Cityinterface/rest/services/OMS.svc/W4_DeleteMenu`, {
      _version: 9999,
      _dc: new Date().getTime(),
      menuID,
      subSystemValue: '手持系统',
    })
      .then(res => {
        if (res.success) {
          setDeleteMenuVisible(false);
          notification.success({
            message: '删除成功',
          });
          updateTrees();
        } else {
          notification.error({
            message: '删除失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        setTreeLoading(false);
        message.error(err);
      });
  };

  return (
    <div className={styles.contentContainer}>
      <div className={styles.menuContainer}>
        {treeData.length > 0 && (
          <Spin spinning={treeLoading} tip="loading...">
            <Tree
              showIcon="true"
              showLine={{ showLeafIcon: false }}
              defaultExpandAll="true"
              // selectedKeys={[currentSelectMenu]}
              // onSelect={onSelect}
              treeData={treeData.map(t => mapTree(t))}
            />
          </Spin>
        )}
      </div>
      <div className={styles.previewContainer}>123</div>
      {/* 必填项提示 */}
      <Modal
        title="提示"
        visible={tipVisible}
        onCancel={() => setTipVisible(false)}
        width="300px"
        centered
        footer={[
          <Button
            key="back"
            type="primary"
            onClick={() => setTipVisible(false)}
          >
            关闭
          </Button>,
        ]}
      >
        <p>
          标记<span className={styles.redText}>*</span>的为必填项
        </p>
      </Modal>
      {/* 添加菜单 */}
      <Modal
        title={menuTitle}
        visible={addMenuVisible}
        onOk={submitAddMenu}
        onCancel={() => setAddMenuVisible(false)}
        okText="确认"
        cancelText="取消"
        centered
      >
        <Form form={addMenuForm}>
          <Form.Item
            name="menuName"
            label="菜单名称"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入菜单名称" />
          </Form.Item>
          <Form.Item
            name="shortName"
            label="菜单别名"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入菜单别名" />
          </Form.Item>
          <Form.Item
            name="imageUrl"
            label="菜单图标"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入菜单图标" />
          </Form.Item>
          <Form.Item
            name="pageUrl"
            label="功能路径"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入功能路径" />
          </Form.Item>
          <Form.Item name="funParam" label="功能参数">
            <Input placeholder="请输入功能参数" />
          </Form.Item>
        </Form>
        <ListCardItem
          itemid="1"
          key="1"
          userList={rolelist}
          OUName="关联角色"
          getValueCallback={getRoleValueCallback}
        />
      </Modal>
      {/* 添加菜单组 */}
      <Modal
        title={menuTitle}
        visible={addMenuGroupVisible}
        onOk={submitAddMenuGroup}
        onCancel={() => setAddMenuGroupVisible(false)}
        okText="确认"
        cancelText="取消"
        centered
      >
        <Form form={addMenuGroupForm}>
          <Form.Item
            name="menuName"
            label="分组名称"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入分组名称" />
          </Form.Item>
          <Form.Item
            name="shortName"
            label="分组别名"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入分组别名" />
          </Form.Item>
          <Form.Item
            name="imageUrl"
            label="菜单图标"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入菜单图标" />
          </Form.Item>
          <Form.Item
            name="pageUrl"
            label="离线图标"
            rules={[{ required: true, message: '不能为空' }]}
          >
            <Input placeholder="请输入离线图标" />
          </Form.Item>
          <Form.Item name="funParam" label="功能参数">
            <Input placeholder="请输入功能参数" />
          </Form.Item>
        </Form>
      </Modal>
      {/* 删除菜单 */}
      <Modal
        title={menuTitle}
        visible={deleteMenuVisible}
        onOk={submitDeleteMenu}
        onCancel={() => setDeleteMenuVisible(false)}
        okText="确认"
        cancelText="取消"
        centered
      >
        <span>确定删除？</span>
      </Modal>
    </div>
  );
};

export default SevenParams;
